"use strict";

Object.defineProperty(exports, "__esModule", { value: true });
const js_1 = require("keccak/js");
const util_1 = require("./util");
class Address {
    constructor(publicKey) {
        this._publicKey = util_1.decompressPublicKey(publicKey);
    }
    static from(publicKey) {
        return new Address(publicKey);
    }
    static checksumAddress(address) {
        const addrLowerCase = address.toLowerCase();
        if (!Address.isValid(addrLowerCase)) {
            throw new Error('invalid address');
        }
        const addr = addrLowerCase.startsWith('0x') ? addrLowerCase.slice(2) : addrLowerCase;
        const hash = js_1.default('keccak256').update(addr, 'ascii').digest('hex');
        let newAddr = '0x';
        for (let i = 0; i < addr.length; i++) {
            if (hash[i] >= '8') {
                newAddr += addr[i].toUpperCase();
            } else {
                newAddr += addr[i];
            }
        }
        return newAddr;
    }
    static isValid(address) {
        const addr = address.match(/^0[xX]/) ? address.slice(2) : address;
        if (addr.length !== 40) {
            return false;
        }
        if (addr.match(/[0-9a-f]{40}/) || addr.match(/[0-9A-F]{40}/)) {
            return true;
        }
        let checksumAddress;
        try {
            checksumAddress = Address.checksumAddress(addr);
        } catch (_err) {
            return false;
        }
        return addr === checksumAddress.slice(2);
    }
    get publicKey() {
        return this._publicKey;
    }
    get rawAddress() {
        if (!this._rawAddress) {
            this._rawAddress = js_1.default('keccak256').update(this._publicKey.slice(1)).digest().slice(-20);
        }
        return this._rawAddress;
    }
    get address() {
        if (!this._address) {
            const rawAddress = this.rawAddress.toString('hex');
            this._address = Address.checksumAddress(rawAddress);
        }
        return this._address;
    }
}
exports.Address = Address;