"use strict";

Object.defineProperty(exports, "__esModule", { value: true });
const elliptic_1 = require("elliptic");
const bignumber_js_1 = require("bignumber.js");
const js_1 = require("keccak/js");
const secp256k1 = new elliptic_1.ec('secp256k1');
function numberToHex(num, includePrefix = true) {
    const hex = new bignumber_js_1.BigNumber(num).toString(16);
    return includePrefix ? '0x' + hex : hex;
}
exports.numberToHex = numberToHex;
function bigNumberToHex(bigNum, includePrefix = true) {
    const hex = bigNum.toString(16);
    return includePrefix ? '0x' + hex : hex;
}
exports.bigNumberToHex = bigNumberToHex;
function hexToEvenLengthHex(hex, includePrefix = true) {
    let h = (hex.match(/^0x/i) ? hex.slice(2) : hex).toLowerCase();
    if (h.length % 2 === 1) {
        h = '0' + h;
    }
    return includePrefix ? '0x' + h : h;
}
exports.hexToEvenLengthHex = hexToEvenLengthHex;
function hexToBuffer(hex) {
    if (hex.length === 0) {
        return Buffer.alloc(0);
    }
    return Buffer.from(hexToEvenLengthHex(hex, false), 'hex');
}
exports.hexToBuffer = hexToBuffer;
function numberToBuffer(num) {
    if (num === 0) {
        return Buffer.alloc(0);
    }
    return hexToBuffer(numberToHex(num, false));
}
exports.numberToBuffer = numberToBuffer;
function bigNumberToBuffer(bigNum) {
    if (bigNum.isZero()) {
        return Buffer.alloc(0);
    }
    return hexToBuffer(bigNumberToHex(bigNum, false));
}
exports.bigNumberToBuffer = bigNumberToBuffer;
function bnToBuffer(bn) {
    if (bn.isZero()) {
        return Buffer.alloc(0);
    }
    return hexToBuffer(bn.toString(16));
}
exports.bnToBuffer = bnToBuffer;
function keccak256(data) {
    const buf = data instanceof Buffer ? data : Buffer.from(data, 'utf8');
    return js_1.default('keccak256').update(buf).digest();
}
exports.keccak256 = keccak256;
function decompressPublicKey(publicKey) {
    const length = publicKey.length;
    const firstByte = publicKey[0];
    if (length !== 33 && length !== 65 || firstByte < 2 || firstByte > 4) {
        throw new Error('invalid public key');
    }
    let key;
    try {
        key = secp256k1.keyFromPublic(publicKey);
    } catch (_err) {
        throw new Error('invalid public key');
    }
    return Buffer.from(key.getPublic().encode());
}
exports.decompressPublicKey = decompressPublicKey;